from langchain.chat_models import ChatOpenAI
from os import getenv


class Chat():

    def __init__(self, prompt_template, **kwargs) :
        self.chat_model = ChatOpenAI(model="gpt-3.5-turbo-0613", **kwargs)
        self.prompt_template = prompt_template

    def generate(self, no_error=True, **input):
        try :
            response = self.chat_model(self.prompt_template.format_messages(**input))
            return response.content
        except Exception as e:
            if no_error:
                print(e)
                return("ERROR")
            else :
                raise e
            


    


from wonkaai.Chat import ChatFunction
from wonkaai.ResultAnalyser import ResultAnalyser
import json

class ChatFunctionMultiple(ChatFunction):

    def __init__(self, prompt_template, n=10, **kwargs):
        super().__init__(prompt_template, n=n, **kwargs)

    def generate(self, function_schema=..., no_error=True, **kwargs):
        response = super().generate(function_schema, no_error, **kwargs)

        if response == "ERROR":
            return None
        
        if response.get("choices") is None:
            return None
        
        choices = response.get("choices")

        result_analyser = ResultAnalyser()
        arguments = []
        for choice in choices :
            message = choice.get("message")
            if message is None :
                result_analyser.add_result({"error" : "no message"})
                continue
            function_call = message.get("function_call")
            if function_call is None :
                result_analyser.add_result({"error" : "no function call"})
                continue   
            name = function_call.get("name")
            arguments =  function_call.get("arguments")
            if name is None or arguments is None :
                result_analyser.add_result({"error" : "no function name"})
                continue
            if name != function_schema[0]["name"] :
                result_analyser.add_result({"error" : "invalid function name"})
                continue
            if arguments is None :
                result_analyser.add_result({"error" : "no function arguments"})
                continue

            try :
                result_analyser.add_result(json.loads(arguments))
            except :
                result_analyser.add_result({"error" : "invalid json"})
                continue
        result_analyser.analyse()
        return result_analyser
                    
    
        



import PyPDF2
import docx

class DocReader:

    def __init__(self, file_path) -> None:
        self.file_path = file_path
        if(self.file_path.endswith('.pdf')):
            self.content = self.load_pdf_text()
        elif(self.file_path.endswith('.doc') or self.file_path.endswith('.docx')):
            self.content = self.load_docs_text()
        

    def load_pdf_text(self):
        with open(self.file_path, 'rb') as file:
            pdf_reader = PyPDF2.PdfReader(file)
            text = ''
            for page in pdf_reader.pages:
                text += page.extract_text()
        return text

    def load_docs_text(file_path):
        doc = docx.Document(file_path)
        # Extract text from paragraphs in the document
        text = '\n'.join([paragraph.text for paragraph in doc.paragraphs])
        return text

    def get_content(self):
        return(self.content)

    


from copy import deepcopy

class ResultAnalyser() :

    def __init__ (self) :
        self.raw = {}
        self.n = 0

    def append_item(self, key, value, raw=None) :
        if raw is None :
            raw = self.raw

        if key not in raw :
            raw[key] = {}

        if isinstance(value, dict) : 
            for k, v in value.items() :
                raw[key] = self.append_item(k, v, raw[key])
            return raw
        else :
            if value in raw[key] :
                raw[key][value] += 1
            else :
                raw[key][value] = 1
            return raw
        
    
    def add_result(self, res) :
        #iterate over dict res
        for key, value in res.items() :
            self.raw = self.append_item(key, value)
        self.n += 1

    def divide_by_n(self, d=None):
        if d is None:
            d = deepcopy(self.raw)

        for key, value in d.items():
            if isinstance(value, dict):
                self.divide_by_n(value)
            else:
                # Check if the leaf value is an integer and non-zero
                if isinstance(value, int) and value != 0:
                    d[key] = value / self.n

        self.results = d


    def get_dict_depth(self, d, depth=0):
        if not isinstance(d, dict) or not d:
            return depth
        else:
            return max(self.get_dict_depth(v, depth + 1) for v in d.values())

    def get_max(self, d = None):
        if d is None:
            d = deepcopy(self.results)

        for key, value in d.items():
            if not isinstance(value, dict):
                continue
            depth = self.get_dict_depth(value)
            if depth == 1:
                d[key] = max(value, key=value.get)
            else:
                self.get_max(value)
        self.results_max = d

    def analyse(self) :
        self.divide_by_n()
        print(self.results)
        self.get_max()

    def to_dict(self) :
        return {
            "results" : self.results,
            "results_max" : self.results_max
        }


from langchain.chat_models import ChatOpenAI
from os import getenv
from langchain.schema import (
    AIMessage,
    HumanMessage,
    SystemMessage
)
import openai
import time


class ChatFunction():

    def __init__(self, prompt_template, **kwargs) :

        if "model" not in kwargs:
            kwargs["model"] = "gpt-3.5-turbo-0613"
        if "temperature" not in kwargs:
            kwargs["temperature"] = 0.7

        self.kwargs = kwargs
        
        self.prompt_template = prompt_template

    def _format_prompt(self, input):
        s = self.prompt_template.format_messages(**input)
        ss = []
        for i in s : 
            if isinstance(i, SystemMessage):
                ss.append({"role" : "system", "content" : i.content})
            elif isinstance(i, HumanMessage):
                ss.append({"role" : "user", "content" : i.content})
            elif isinstance(i, AIMessage):
                ss.append({"role" : "assistant", "content" : i.content})
            
        return(ss)

    def generate(self, function_schema = [], no_error=True, retry=5, **kwargs):
        try :
            if retry < 0 :
                raise ValueError("Too many retries")
            if len(function_schema) == 0  :
                raise ValueError("No function schema")
            response = openai.ChatCompletion.create(
                **self.kwargs,
                messages=self._format_prompt(kwargs),
                functions=function_schema,
                function_call={"name" : function_schema[0]["name"]},
            ) 
            return response
        except openai.error.RateLimitError as e:
            print("Rate Limit Error, retrying in {} seconds".format(4*(5-retry+1)))
            time.sleep(4*(5-retry+1))
            self.generate(function_schema, no_error, retry=retry-1, **kwargs)
        except openai.error.ServiceUnavailableError as e :
            print("Rate Limit Error, retrying in {} seconds".format(4*(5-retry+1)))
            time.sleep(4*(5-retry+1))
            self.generate(function_schema, no_error, retry=retry-1, **kwargs)
            raise openai.error.RateLimitError("Unstable Error: " + str(e)) from e
        
        except Exception as e:
            if no_error:
                print(e)
                return("ERROR")
            else :
                raise e